//		Copyright (c) 1999 by
//		Advanced Visual Systems Inc.
//		All Rights Reserved
//	
//	This software comprises unpublished confidential information of
//	Advanced Visual Systems Inc. and may not be used, copied or made
//	available to anyone, except in accordance with the license
//	under which it is furnished.
//	
//      This file is under Perforce control
//      $Id: //depot/express/dbproj/dbil/sdi_query.hxx#1 $
//

//
// file: sdi_query.hxx
// author: Patrick Li, Advanced Visual System
// purpose: Define SDI_Query class
//
//  This class opens and maintains a query against the spatial engine specified by 
//  the SDI_Connection. The query remains open until all the rows are fetched.
//
#ifndef SDI_QUERY_DEFINED
#define SDI_QUERY_DEFINED

#include "sdi_connect.hxx"
#include "sdi_shp_arr.hxx"

// Define SDI_Query class
//
class SDI_Query : public SDI_Error_Tool {
public:
   // Constructor and Destructor
   //
   SDI_Query();
   virtual ~SDI_Query();

   // Setup a basic SQL query
   //
   SDI_Status Setup( SDI_Connection *in_connection, char *sql_statement, int rows_per_fetch,
							int userbuf_realloc_interval, int max_rows);
/*   
   // Setup a spatial query
   //
   SDI_Status Setup( SDI_Connection *in_connection, int num_tables, char **tables, 
                     char *where, char *shape, char *field, int num_attrs, char **attributes, 
                     SDI_Shape *filter_shape, SDI_FILTER_METHODS filter_method );
*/
	SDI_Status Execute( SDI_Connection *in_connection, int iteration );
	SDI_Status Define( SDI_Connection *in_connection );

   // Setup Fetch parameters
   // 
   SDI_Status SetupFetch( int in_rows_per_fetch, int in_fetch_all, int in_max_rows );
   SDI_Status SetRowsPerFetch( int in_rows_per_fetch );
   SDI_Status SetLoop( int in_loop );
   SDI_Status SetFetchAll( int in_fetch_all );
   SDI_Status SetMaxRows( int in_max_rows );
	SDI_Status SetFlushWhenFull( int in_flush_when_full );
	SDI_Status SetQueryTimeoutSecs( int in_query_timeout_secs );
	SDI_Status SetUserbufReallocInterval(int in_userbuf_realloc_interval );
	SDI_Status SetUseFloatingTypeOnly(int in_use_floating_type_only );
	SDI_Status SetFloatingType(int in_floating_type );
	SDI_Status SetFixedType(int in_fixed_type );
	SDI_Status SetRowSpecificErrorCnt( int in_row_specific_error_cnt );
   SDI_Status SetNullValue( unsigned char null_value );
   SDI_Status SetNullValue( short null_value );
   SDI_Status SetNullValue( int null_value );
   SDI_Status SetNullValue( unsigned int null_value );
   SDI_Status SetNullValue( float null_value );
   SDI_Status SetNullValue( double null_value );
   SDI_Status SetNullValue( char* null_value );
   // Fetch attributes, shapes, and fields from current query
   //
//   SDI_Status Fetch( SDI_Shape *shape );
//   SDI_Status Fetch( SDI_Shape_Array *shapes, SDI_Status_Callback cb = NULL );
   SDI_Status Fetch( int *num_attrs, SDI_Attribute **attrs,  
		void* cb = NULL, signed char **in_null_ind = NULL, void* ptr = NULL, short new_fetch = 1);

   // Check to see if there is a valid query
   //
   int IsValid();

	// Release user buffers 
	void ReleaseUserBuffer();
	// Release space if prepareStatement/query failed or deinstanced 
	void CloseCursor();

   // Methods to get parameters about the current query
   //
   const char *GetTable() { return cursor.table; }
   const char *GetShapeName() { return cursor.shape_name; }
   const char *GetFieldName() { return cursor.field_name; }
   int GetNumAttributes() { return cursor.num_attrs; }
	int GetRowSpecificError_cnt(){return cursor.output_stat.row_specific_error_cnt;}
   SDI_Status SetAttrNulls( int num_attrs, SDI_Attribute *attrs);

	const DB_OUTPUT_STAT *GetOutputStatistics() const {return &(cursor.output_stat);}
	void ResetOutputStatistics();
	long **GetNullIndicators() const {return cursor.indicators;}
   void SigHandler ();
private:
   // Connection and cursor information
   //
   SDI_Cursor cursor;
   SDI_Connection *connection;

   // Fetch parameters
   //
	int flush_when_full;
	int query_timeout_secs;
	int userbuf_realloc_interval;
   int fetch_all;
   int max_rows;
	int loop;
	int cancel_button;
	short new_statement;

	// User buffers
	SDI_Attribute * attrs;
	signed char * null_ind;

	// Allocate space for the pointer arrays
	SDI_Status SDI_InitBuffers(int num_attrs); 

   // Create private copy constructor and assignment operator 
   // to invalidate copys and assignments
   //
   SDI_Query( const SDI_Query & ) {}
   SDI_Query &operator=(const SDI_Query &) { return *this; }
	void SetAlarm();
	SDI_Status CancelAlarm();
};
#endif
